if(!TESTING)
{
    System.Link("StyleSheet", Module.GetResourceUrl("BBS.css"), "text/css");
}

var Controls = null, CommonDialog = null, IO = null, Communication = null, Management = null;
var Window = null, Control = null, RichEditor = null;

function init(completeCallback, errorCallback)
{
    function LoadModulesComplete()
    {
        IO = System.GetModule("IO.js");
        Communication = System.GetModule("Communication.js");
        Management = System.GetModule("Management.js");
        Controls = System.GetModule("Controls.js");
        CommonDialog = System.GetModule("CommonDialog.js");

        Window = System.GetModule("Window.js").Window;
        Control = System.GetModule("Controls.js").Control;
        
        RichEditor = System.GetModule("RichEditor.js").RichEditor;

        _init(completeCallback, errorCallback);
    }

    System.LoadModules(
        LoadModulesComplete,
        errorCallback,
        ["IO.js", "Communication.js", "Management.js", "Window.js", "Controls.js", "CommonDialog.js", "RichEditor.js"]
    );
}

function _init(completeCallback, errorCallback)
{
    try
    {
        //初始化代码，初始化完成后必须调用completeCallback;
        GetAllCategory(completeCallback, errorCallback);
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}



function dispose(completeCallback, errorCallback)
{
    _dispose(completeCallback, errorCallback);
}

function _dispose(completeCallback, errorCallback)
{
    try
    {
        //卸载代码，卸载完成后必须调用completeCallback;
        completeCallback();
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}

//共享全局变量和函数，在此定义的变量和函数将由该应用程序的所有实例共享
function EmptyCallback()
{

}

//格式化数字
function FormatNumber(num, length)
{
    var s = num.toString();
    var zero = "";
    for (var i = 0; i < length - s.length; i++) zero += "0";
    return zero + s;
}

//时间转字符串
function DateToString(date)
{
    return String.format("{0}-{1}-{2} {3}:{4}:{5}", FormatNumber(date.getFullYear(), 4), FormatNumber(date.getMonth() + 1, 2), FormatNumber(date.getDate(), 2), FormatNumber(date.getHours(), 2), FormatNumber(date.getMinutes(), 2), FormatNumber(date.getSeconds(), 2));
}

//时间转字符串
function TimeToString(date)
{
    return String.format("{0}:{1}:{2}", FormatNumber(date.getHours(), 2), FormatNumber(date.getMinutes(), 2), FormatNumber(date.getSeconds(), 2));
}


var Instance = null;
var Categories = {};
var CurrentCategoryID = 11;
var Mode = 1;

function RemoveThemeListener(completeCallback, errorCallback, cmdId, categoryId)
{
    var params = {
        CommandID: cmdId,
        CategoryID: categoryId
    };
    var data = System.CreateServiceData("BBS_Service", "RemoveThemeListener", params);

    System.SendCommand(RemoveThemeListener_Complete, errorCallback, data, true);

    function RemoveThemeListener_Complete(ret)
    {
        completeCallback(ret);
    }
}

function AddThemeListener(completeCallback, errorCallback, cmdId, categoryId, receive)
{
    var params = {
        CommandID: cmdId,
        CategoryID: categoryId,
        ReceiveThemes: IsNull(receive, true)
    };
    var data = System.CreateServiceData("BBS_Service", "AddThemeListener", params);

    System.SendCommand(AddThemeListener_Complete, errorCallback, data, true);

    function AddThemeListener_Complete(ret)
    {
        completeCallback(ret);
    }
}

function GetAllCategory(completeCallback, errorCallback)
{
    var params = {};
    var data = System.CreateServiceData("BBS_Service", "GetAllCategory", params);

    System.SendCommand(GetAllCategory_Complete, errorCallback, data, true);

    function GetAllCategory_Complete(ret)
    {
        for (var i in ret.Categories)
        {
            var c = ret.Categories[i];
            c.SubCategories = [];
            Categories[c.ID] = c;
        }
        for (var i in ret.Categories)
        {
            var c = ret.Categories[i];
            if (c.Parent != 0) Categories[c.Parent].SubCategories.push(c);
        }
        completeCallback(ret);
    }
}

function NewTheme(completeCallback, errorCallback, categoryId, subject, content)
{
    var params = {
        Creator: System.GetUser(),
        Subject: subject,
        Content: content,
        CategoryID: categoryId
    };
    var data = System.CreateServiceData("BBS_Service", "NewTheme", params);

    System.SendCommand(NewTheme_Complete, errorCallback, data, true);

    function NewTheme_Complete(ret)
    {
        completeCallback(ret);
    }
}

function RemoveMessageListener(completeCallback, errorCallback, cmdId, themeId)
{
    var params = {
        CommandID: cmdId,
        ThemeID: themeId
    };
    var data = System.CreateServiceData("BBS_Service", "RemoveMessageListener", params);

    System.SendCommand(RemoveMessageListener_Complete, errorCallback, data, true);

    function RemoveMessageListener_Complete(ret)
    {
        completeCallback(ret);
    }
}

function AddMessageListener(completeCallback, errorCallback, cmdId, themeId, receive)
{
    var params = {
        CommandID: cmdId,
        ThemeID: themeId,
        ReceiveMessages: IsNull(receive, true)
    };
    var data = System.CreateServiceData("BBS_Service", "AddMessageListener", params);

    System.SendCommand(AddMessageListener_Complete, errorCallback, data, true);

    function AddMessageListener_Complete(ret)
    {
        completeCallback(ret);
    }
}

function NewMessage(completeCallback, errorCallback, themeId, content)
{
    var params = {
        Sender: System.GetUser(),
        Content: content,
        ThemeID: themeId
    };
    var data = System.CreateServiceData("BBS_Service", "NewMessage", params);

    System.SendCommand(NewMessage_Complete, errorCallback, data, true);

    function NewMessage_Complete(ret)
    {
        completeCallback(ret);
    }
}

function GetThemeContent(completeCallback, errorCallback, themeId)
{
    var params = {
        ThemeID: themeId
    };
    var data = System.CreateServiceData("BBS_Service", "GetThemeContent", params);

    System.SendCommand(GetThemeContent_Complete, errorCallback, data, true);

    function GetThemeContent_Complete(ret)
    {
        completeCallback(ret);
    }
}

//消息处理
function TranslateMessage(msg)
{
    msg = msg.replace(/<img [^\t\n\r\f\v<>]+>/ig,
    function(img)
    {
        return img.replace(/src=\x22[^\t\n\r\f\v\x22]+\x22/ig,
        function(src)
        {
            return src.replace(/download.aspx\x3FFileName=([^\x22\s]+)/ig,
            function(filename, path)
            {
                return String.format("download.aspx?FileName={{Accessory type=\"image\" src=\"{0}\"}", path);
            })
        });
    });
    msg = msg.replace(/<a [^\t\n\r\f\v<>]+[^\x2F]>/ig,
    function(a)
    {
        var hrefReg = /href=\x22[^\t\n\r\f\v\x22]+\x22/ig
        hrefReg.lastIndex = 0;
        var href = hrefReg.exec(a);
        return String.format("[A:{0}]", href != null && href.length > 0 ? escape(href[0].substr(6, href[0].length - 7)) : "");
    });
    msg = msg.replace(/<\x2Fa>/ig,
    function(a)
    {
        return "[/A]";
    });
    msg = msg.replace(/\x5BFILE\x3A[^\n\f\r\t\v]+\x5D/g,
    function(file)
    {
        return String.format("[FILE:{{Accessory type=\"file\" src=\"{0}\"}]", escape(file.substr(6, file.length - 7)));
    });
    msg = msg.replace(/<!--[^\n\f\r\t\v<>]*-->/g,
    function()
    {
        return "";
    });
    return msg;
}

function Application()
{
    var CurrentApplication = this;
    var m_MainForm = null;
    
    //应用程序全局对象;
    this.StartChat = function(theme)
    {
        m_MainForm.StartChat(theme);
    }
    
    this.Show = function(isTop)
    {
        m_MainForm.Show(isTop);
    }    

    this.Start = function(baseUrl)
    {
        //应用程序入口;
        Mode = IsNull(arguments[1], 1);
        
        if (Instance == null)
        {
            m_MainForm = new MainForm();
            m_MainForm.OnClosed.Attach(
            function()
            {
                CurrentApplication.Dispose();
                Instance = null;
            });
            m_MainForm.Show(true);
            m_MainForm.Maximun();
            Instance = CurrentApplication;
        }
        else
        {
            Instance.Show(true);
            CurrentApplication.Dispose();
        }
    }

    this.Terminate = function(completeCallback, errorCallback)
    {
        try
        {
            //应用程序终止，退出系统时用系统调用;
            completeCallback();
        }
        catch (ex)
        {
            errorCallback(new Exception(ex.mame, ex.message));
        }
    }
    function MainForm()
    {
        var This = this;
        var OwnerForm = this;
        
        var config = {"Left":6,"Top":6,"Width":643,"Height":524,"AnchorStyle":Controls.AnchorStyle.Left | Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":true,"HasMaxButton":true,"Resizable":true,"Title":{"InnerHTML":"主窗口"}};
        if (!TESTING && Mode == 0)
        {
            config.Left = 0;
            config.Top = 0;
            config.Width -= 6 * 2;
            config.Height -= 6 * 2 + 18;
            config.BorderWidth = 0;
            config.Title.Height = 0;
            config.HasMinButton = false;
            config.ControlMode = true;
            config.BkImage = {
                Horiz: [6, 100, 6],
                Vertical: [24, 100, 6],
                Css: "BBS_WindowBk"
            };
        }
    
        Window.call(This, config);
    
        var Base = {
            GetType: This.GetType,
            is: This.is
        };
    
        This.GetType = function() { return "MainForm"; }
        This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
        
        var control3 = new Controls.Control({"Left":6,"Top":28,"Width":296,"Height":459,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"});
        
        
        
        control3.OnResized.Attach(
            function(btn)
            {
                
            }
        )

        
        var m_CategoryTree_Config={"Left":0,"Top":0,"Width":296,"Height":322,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":control3,"Text":"","Css":"control"};
        
                m_CategoryTree_Config.BorderWidth = 1;
        
        var m_CategoryTree = new CategoryTree(m_CategoryTree_Config);
        
        

        
        var m_CategoryList_Config={"Left":0,"Top":329,"Width":296,"Height":130,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":control3,"Text":"","Css":"control"};
        
                m_CategoryList_Config.BorderWidth = 1;
        
        var m_CategoryList = new CategoryList(m_CategoryList_Config);
        
        

        
        var m_Toolbar = new Controls.Toolbar({"Left":6,"Top":2,"Width":619,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"toolbar","Items":[{"Text":"修改个人资料","Css":"Image22_Option","Command":"Setting"},{"Text":"发帖","Css":"Image22_Add","Command":"NewTheme"},{"Text":"意见反馈","Css":"Image22_MessageRecord","Command":"Admin"}]});
        
        m_Toolbar.SetButtonVisible(0,false);
        
        m_Toolbar.OnCommand.Attach(
            function(command)
            {
                if (command == "NewTheme")
                {
                    if (CurrentCategoryID == 0)
                    {
                        CommonDialog.Alert("请选择一个类别！", This);
                        return;
                    }
                    if (!Categories[CurrentCategoryID].EnableCreateTheme)
                    {
                        CommonDialog.Alert("新闻区不允许新建主题！", This);
                        return;
                    }
                    var form = new NewThemeForm();
                    form.Resize(This.GetClientWidth() - 100, This.GetClientHeight() - 100);
                    form.ShowDialog(This, 'center', 0, 0);
                }
                else if (command == "Setting")
                {
                    System.Exec(EmptyCallback, EmptyCallback, "Setting/Setting.js");
                }
                else if (command == "Admin")
                {
                    System.Exec(EmptyCallback, EmptyCallback, "IM/IM.js", "administrator", true);
                }
            }
        )

        
        var m_ChatPanelContainer_Config={"Left":309,"Top":28,"Width":316,"Height":459,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
        
                
        
        var m_ChatPanelContainer = new ChatPanelContainer(m_ChatPanelContainer_Config);
        
        
    
    
        var m_Task = null;
        if(config.HasMinButton)
        {
            m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
            This.OnClosed.Attach(
                function()
                {
                    Taskbar.RemoveTask(m_Task);
                }
            );
        }
        Controls.CreateHorizSplit(m_CategoryTree, m_CategoryList);
        Controls.CreateVerticalSplit(control3, m_ChatPanelContainer);
        
        function m_CategoryTree_OnClick()
        {
            var node = m_CategoryTree.GetSelectedNode();
            if (node != null && node.GetTag() != null)
            {
                var c = node.GetTag();
                if (c.Type == 1)
                {
                    CurrentCategoryID = c.ID;
                    //m_Toolbar.SetButtonVisible(1, Categories[CurrentCategoryID].EnableCreateTheme);
                    m_CategoryList.Load(c.ID);
                }
            }
        }
        
        This.StartChat = function(theme)
        {
            m_ChatPanelContainer.StartChat(theme);
        }
        
        m_CategoryTree.OnClick.Attach(m_CategoryTree_OnClick);
        
        if (TESTING)
        {
            This.MoveEx('center', 0, 0);
        }
        else
        {
            This.MoveEx('left|top', 0, 0);
        }
        
        function m_CategoryList_OnClick(theme)
        {
            m_ChatPanelContainer.StartChat(theme);
        }
        
        CurrentCategoryID = 11;
        
        m_CategoryList.OnClick.Attach(m_CategoryList_OnClick);
        
        m_CategoryList.Load(CurrentCategoryID);
        
        //m_Toolbar.SetButtonVisible(1, Categories[CurrentCategoryID].EnableCreateTheme);
        
    }

}

function CategoryNodeSort(c1, c2)
{
    if (c1.Tag.Type < c2.Tag.Type) return 1;
    if (c1.Tag.Type > c2.Tag.Type) return - 1;
    if (c1.Tag.SEQ < c2.Tag.SEQ) return 1;
    if (c1.Tag.SEQ > c2.Tag.SEQ) return - 1;
    return 0;
}

var CategoryTreeDS = (function()
{
    var obj = {};
    obj.GetSubNodes = function(callback, node)
    {
        if (node == null)
        {
            var nodes = [{
                Name: "YunXiangBBS",
                Text: "云翔在线聊天室",
                Tag: null,
                ImageCss: "Image16_Folder"
            }];
            callback(nodes);
        }
        else if (node.GetFullPath() == "/YunXiangBBS")
        {
            var nodes = [];
            for (var key in Categories)
            {
                var c = Categories[key];
                if (c.Parent == 0)
                {
                    var node = {
                        Name: c.ID,
                        Text: c.Name,
                        Tag: c,
                        ImageCss: c.SubCategories.length > 0 ? "Image16_Folder": "Image16_File",
                        HasChildren: c.SubCategories.length > 0
                    };
                    nodes.push(node);
                }
            }
            nodes.sort(CategoryNodeSort);
            callback(nodes);
        }
        else
        {
            var nodes = [];
            var tag = node.GetTag();
            for (var i in tag.SubCategories)
            {
                var c = tag.SubCategories[i];
                var node = {
                    Name: c.ID,
                    Text: c.Name,
                    Tag: c,
                    ImageCss: c.SubCategories.length > 0 ? "Image16_Folder": "Image16_File",
                    HasChildren: c.SubCategories.length > 0
                };
                nodes.push(node);
            }
            nodes.sort(CategoryNodeSort);
            callback(nodes);
        }
    }

    return obj;
})();

function CategoryTree(config)
{
    var This = this;
    var OwnerForm = this;
    
config.DataSource = CategoryTreeDS;
    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    Controls.TreeView.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "CategoryTree"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.Refresh(EmptyCallback);
    This.Expand(EmptyCallback, "/YunXiangBBS");
    This.Expand(EmptyCallback, "/YunXiangBBS/1");
    This.Expand(EmptyCallback, "/YunXiangBBS/20");
    This.Select(EmptyCallback, "/YunXiangBBS/1/11");

    This.Resize(width,height);
}


function ThemePanel(list, themeInfo)
{
    var temp = [];
    temp.push('<table class="BSS_Theme" cellspacing="4" cellpadding="0" style="width:100%;">');
    temp.push('<tr>');
    temp.push('<td class="BSS_Theme_Text"></td>');
    temp.push('</tr>');
    temp.push('<tr>');
    temp.push('<td class="BSS_Theme_Footer"><div class="BBS_Theme_Creator"></div></td>');
    temp.push('</tr>');
    temp.push('</table>');

    var dom = document.createElement("DIV");
    dom.innerHTML = temp.join("");

    var domSubject = dom.firstChild.rows[0].cells[0];
    var domCreator = dom.firstChild.rows[1].cells[0].childNodes[0];

    this.GetDom = function()
    {
        return dom;
    }

    dom.onclick = function()
    {
        list.OnClick.Call(themeInfo);
    }

    domSubject.innerHTML = System.ReplaceHtml(themeInfo.Subject);

    var info = "";

    if (themeInfo.CreatorName != "administrator")
    {
        info += String.format("{0} ", themeInfo.CreatorNickname);
    }

    var ct = themeInfo.CreatedTime,
    now = new Date();
    if (ct.getFullYear() == now.getFullYear() && ct.getMonth() == now.getMonth() && ct.getDate() == now.getDate())
    {
        info += "创建于 " + TimeToString(ct);
    }
    else
    {
        info += "创建于 " + DateToString(ct);
    }
    domCreator.innerHTML = System.ReplaceHtml(info);
}

function CategoryList(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css="BBS_CategoryList"
    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "CategoryList"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    var m_Theme = config.Theme;
    
    This.OnClick = new Delegate();
    
    this.GetDom().style.overflowY = "auto";
    
    this.Insert = function(themeInfo)
    {
        var panel = new ThemePanel(This, themeInfo);
        if (This.GetDom().childNodes.length == 0)
        {
            This.GetDom().appendChild(panel.GetDom());
        }
        else
        {
            This.GetDom().insertBefore(panel.GetDom(), This.GetDom().firstChild);
        }
    }
    
    var m_CommandID = -1;
    var m_CategoryID = 0;
    
    function Theme_Sort(t1, t2)
    {
        if (t1.SEQ < t2.SEQ) return 1;
        if (t1.SEQ > t2.SEQ) return - 1;
    
        if (t1.CreatedTime > t2.CreatedTime) return 1;
        if (t1.CreatedTime < t2.CreatedTime) return - 1;
        return 0;
    }
    
    function AddThemeListener_Complete(ret)
    {
        This.Completed();
        This.Clear();
        ret.Themes.sort(Theme_Sort);
        for (var i in ret.Themes)
        {
            This.Insert(ret.Themes[i]);
            if (ret.Themes[i].ID == Categories[m_CategoryID].DefTheme)
            {
                Instance.StartChat(ret.Themes[i]);
            }
        }
    }
    
    function ErrorCallback(ex)
    {
        This.Completed();
        CommonDialog.Alert(ex.toString(), This.GetTopContainer());
    }
    
    function ListenerHandler(data)
    {
        This.Insert(data.Theme);
    }
    
    This.Load = function(categoryId)
    {
        This.GetDom().scrollTop = 0;
        if (m_CategoryID == categoryId)
        {
            This.Waiting();
            AddThemeListener(AddThemeListener_Complete, ErrorCallback, m_CommandID, categoryId);
        }
        else
        {
            if (m_CommandID != -1)
            {
                System.RemoveCommandHandler(m_CommandID);
                RemoveThemeListener(EmptyCallback, ErrorCallback, m_CommandID, categoryId);
            }
    
            This.Waiting();
            m_CommandID = System.GetUser().toUpperCase() + "-BBS-" + System.GenerateUniqueId();
            m_CategoryID = categoryId;
            System.RegisterCommandHandler(m_CommandID, ListenerHandler);
            AddThemeListener(AddThemeListener_Complete, ErrorCallback, m_CommandID, categoryId);
        }
    }
    
    This.Clear = function()
    {
        This.GetDom().removeChild(m_WaitingDom);
        This.GetDom().innerHTML = "";
        This.GetDom().appendChild(m_WaitingDom);
    }
    
    System.OnSessionRecover.Attach(function()
    {
        if (m_CategoryID != 0)
        {
            AddThemeListener(AddThemeListener_Complete, ErrorCallback, m_CommandID, m_CategoryID);
        }
    });
    
    var m_WaitingDom = document.createElement("DIV");
    m_WaitingDom.className = "BBS_CategoryList_Waiting";
    m_WaitingDom.style.left = "0px";
    m_WaitingDom.style.right = "0px";
    m_WaitingDom.style.top = "0px";
    m_WaitingDom.style.position = "absolute";
    m_WaitingDom.style.zIndex = 100;
    m_WaitingDom.style.display = "none";
    m_WaitingDom.innerHTML = "正在加载...";
    
    This.GetDom().appendChild(m_WaitingDom);
    
    This.Waiting = function()
    {
        m_WaitingDom.style.display = "block";
    }
    This.Completed = function()
    {
        m_WaitingDom.style.display = "none";
    }

    This.Resize(width,height);
}




function ChatPanel(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=446;
    config.Height=418;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "ChatPanel"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_MsgEditor_Config={"Left":0,"Top":296,"Width":446,"Height":86,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            m_MsgEditor_Config.BorderWidth = 1;
    
    var m_MsgEditor = new MessageEditor(m_MsgEditor_Config);
    
    

    
    var m_MsgPanel_Config={"Left":0,"Top":25,"Width":446,"Height":266,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            m_MsgPanel_Config.BorderWidth = 1;
            m_MsgPanel_Config.Operation = [{
                Text: "引用",
                Command: "Quote"
            },{
                Text: "加为好友",
                Command: "Add"
            },{
                Text: "私聊",
                Command: "Chat"
            }];
    
    var m_MsgPanel = new MessagePanel(m_MsgPanel_Config);
    
    function MessagePanel_OnCommand(cmd, msg, msgHtml)
    {
        if (cmd == "Quote")
        {
            m_MsgEditor.Append(String.format("<div class='message_quote'>" + "<div class='messageTitle'>" + "<span class='sender'>{0}</span>" + "<span class='time'>{1}</span>" + "</div>" + "<div class='messageContent'>{2}</div>" + "</div><br/>", msg.SenderNickname, DateToString(msg.CreatedTime), msg.Content));
        }
        else if (cmd == "Chat")
        {
            System.Exec(EmptyCallback, EmptyCallback, "IM/IM.js", msg.SenderName);
        }
        else if (cmd == "Add")
        {
            var arg = {
                Command: "AddFriend",
                Name: msg.SenderName,
                Nickname: msg.SenderNickname,
                ID: msg.Sender
            };
            System.Exec(EmptyCallback, EmptyCallback, "IM/IM.js", arg);
        }
    }
    
    m_MsgPanel.OnCommand.Attach(MessagePanel_OnCommand);

    
    var m_BtnSend = new Controls.Button({"Left":307,"Top":390,"Width":137,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"发送(Ctrl+Enter)","Css":"button_default"});
    
    
    
    m_BtnSend.OnClick.Attach(
        function(btn)
        {
            if (m_MsgEditor.GetValue().length > 10240)
            {
                CommonDialog.Alert("消息不能超过10240个字符！", This.GetTopContainer());
                return;
            }
            
            var content = TranslateMessage(m_MsgEditor.GetValue());
            
            var temp = m_MsgPanel.CreateMessage(content);
            
            m_MsgEditor.SetValue(" ");
            m_MsgEditor.Focus();
            
            This.SetAutoScroll(true);
            
            NewMessage(
            function(ret)
            {
                m_MsgPanel.AddMessage(ret.Message, temp);
            },
            function(ex)
            {
                m_MsgPanel.ShowError(ex.Message, temp);
            },
            config.Theme.ID, content);
        }
    )

    
    var m_Toolbar = new Controls.Toolbar({"Left":0,"Top":0,"Width":446,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"toolbar","Items":[{"Text":"查看主帖","Css":"Image22_File","Command":"ViewContent"},{"Text":"关闭","Css":"Image22_Delete","Command":"Close"}]});
    
    
    
    m_Toolbar.OnCommand.Attach(
        function(command)
        {
            if (command == "Close")
            {
                Stop();
                config.Container.EndChat(config.Theme.ID);
            }
            else if (command == "ViewContent")
            {
                This.SetAutoScroll(!m_MsgPanel.GetAutoScroll());
            }
        }
    )

    
    var m_LabelUserCount = new Controls.Label({"Left":6,"Top":394,"Width":290,"Height":17,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"BBS_Label_UserCount"});
    
    
    

    var m_Split = Controls.CreateHorizSplit(m_MsgPanel, m_MsgEditor, false);
    
    if (config.Theme.Content == "")
    {
        var content = String.format('<div class="news"><img src="{0}"/></div>', Module.GetResourceUrl("waiting.gif"));
    
        temp = m_MsgPanel.CreateContent(content, null, false, false);
    
        GetThemeContent(
        function(ret)
        {
            if (ret.Result == "OK")
            {
                config.Theme.Content = ret.Content;
                var msg = {
                    Content: '<div class="news">' + config.Theme.Content + '</div>',
                    CreatedTime: config.Theme.CreatedTime,
                    SenderNickname: config.Theme.CreatorNickname
                }
    
                m_MsgPanel.AddMessage(msg, temp, false, false);
            }
        },
        function(error)
        {
            CommonDialog.Alert(error.toString(), This.GetTopContainer());
        },
        config.Theme.ID);
    }
    else
    {
    
        var msg = {
            Content: '<div class="news">' + config.Theme.Content + '</div>',
            CreatedTime: config.Theme.CreatedTime,
            SenderNickname: config.Theme.CreatorNickname
        }
    
        m_MsgPanel.AddMessage(msg, null, false, false);
    }
    
    var m_CommandID = System.GetSessionID() + "-" + System.GenerateUniqueId();
    
    function MessageListnerHandler(ret)
    {
        if (ret.Message.Sender != System.GetUserInfo().ID)
        {
            m_MsgPanel.AddMessage(ret.Message)
        }
    
        m_LabelUserCount.SetText(String.format("共有 {0} 人关注", ret.Count));
    }
    
    m_MsgEditor.OnKeyDown.Attach(
    function(evt)
    {
        if (evt.keyCode == 13 && evt.ctrlKey && !evt.shiftKey && !evt.altKey)
        {
            m_BtnSend.Click();
        }
    });
    
    function Msg_Sort(t1, t2)
    {
        if (t1.CreatedTime > t2.CreatedTime) return 1;
        if (t1.CreatedTime < t2.CreatedTime) return - 1;
        return 0;
    }
    
    function AddListener()
    {
        AddMessageListener(
        function(ret)
        {
            ret.Messages.sort(Msg_Sort);
            for (var i in ret.Messages)
            {
                m_MsgPanel.AddMessage(ret.Messages[i])
            }
            m_LabelUserCount.SetText(String.format("共有 {0} 人关注", ret.Count));
        },
        function(ex)
        {
            CommonDialog.Alert(ex.toString(), This.GetTopContainer());
        },
        m_CommandID, config.Theme.ID);
    }
    
    function Stop()
    {
        System.RemoveCommandHandler(m_CommandID);
        RemoveMessageListener(EmptyCallback, EmptyCallback, m_CommandID, config.Theme.ID);
    }
    
    This.Stop = Stop;
    
    This.GetThemeInfo = function()
    {
        return config.Theme;
    }
    
    This.SetAutoScroll = function(scroll)
    {
        if (scroll)
        {
            m_Toolbar.SetButtonText(0, config.Theme.CategoryID <= 2 ? "查看新闻原文": "查看帖子内容");
            m_MsgPanel.SetAutoScroll(true);
            m_MsgPanel.ScrollToBottom();
        }
        else
        {
            m_Toolbar.SetButtonText(0, config.Theme.CategoryID <= 2 ? "评论自动滚动": "回复自动滚动");
            m_MsgPanel.SetAutoScroll(false);
            m_MsgPanel.ScrollToTop();
        }
    }
    
    function ShowComment(visible)
    {
        if (m_MsgEditor.GetVisible() == visible) return;
    
        var diff = m_BtnSend.GetTop() + m_BtnSend.GetHeight() - m_Split.GetTop();
    
        if (visible)
        {
            m_MsgPanel.Resize(m_MsgPanel.GetWidth(), m_MsgPanel.GetHeight() - diff);
            m_MsgEditor.SetVisible(true);
            m_Split.SetVisible(true);
            m_BtnSend.SetVisible(true);
            m_LabelUserCount.SetVisible(true);
        }
        else
        {
            m_MsgPanel.Resize(m_MsgPanel.GetWidth(), m_MsgPanel.GetHeight() + diff);
            m_MsgEditor.SetVisible(false);
            m_Split.SetVisible(false);
            m_BtnSend.SetVisible(false);
            m_LabelUserCount.SetVisible(false);
        }
    }
    
    if (config.Theme.EnableComment)
    {
        System.RegisterCommandHandler(m_CommandID, MessageListnerHandler);
        AddListener();
        System.OnSessionRecover.Attach(function()
        {
            AddMessageListener(EmptyCallback, ErrorCallback, m_CommandID, config.Theme.ID, false);
        });
    }
    
    This.SetAutoScroll(false);
    m_LabelUserCount.SetVisible(config.Theme.EnableComment);
    m_Toolbar.SetButtonVisible(0, config.Theme.EnableComment);
    ShowComment(config.Theme.EnableComment);

    This.Resize(width,height);
}




function MessagePanel(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "BBS_MessagePanel";
    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    Controls.Frame.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "MessagePanel"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.Link("StyleSheet", Module.GetResourceUrl("EditorCss.css"), "text/css");
    
    var m_FileHtmlFormat = "<div class='div_filename'>文件名:{0}</div>" + "<div class='operationContainer'>" + "<div class='link_download'><a target='_blank' href='download.aspx?FileName={1}'>下载</a></div>" + "<div class='link_saveas'><span onclick='javascript:window.SaveFile(unescape(\"{1}\"))'>另存为</span></div>" + "<div class='link_open'><span onclick='javascript:window.OpenFile(unescape(\"{1}\"))'>打开</span></div>" + "</div>";
    
    This.OnCommand = new Delegate();
    
    This.Clear = function()
    {
        This.GetDocument().body.innerHTML = "";
    }
    
    This.CreateContent = function(content)
    {
        var msgDiv = This.GetDocument().createElement("DIV");
        msgDiv.className = "message";
        msgDiv.innerHTML = String.format("<div class='messageContent'>{0}</div>", content);
        This.GetDocument().body.appendChild(msgDiv);
        if (m_AutoScroll) msgDiv.scrollIntoView();
        return msgDiv;
    }
    
    This.CreateMessage = function(content)
    {
        content = content.replace(/<img\s[^<>]*>/ig,
        function(val)
        {
            return String.format("<img src='{0}'/>", Module.GetResourceUrl("loading_img.gif"));
        });
    
        content = content.replace(/\x5BFILE\x3A\s*\{Accessory\s+type=\"file\"\s+src=\"([^\t\n\r\f\v\x5B\x5D]+)\"\}\x5D/ig,
        function(val, file)
        {
            return String.format("<div class='div_filename'><img src='{0}'/></div>", Module.GetResourceUrl("loading_file.gif"), file);
        });
    
        var msgDiv = This.GetDocument().createElement("DIV");
        msgDiv.className = "message";
        msgDiv.innerHTML = String.format("<div class='messageTitle'>" + "<span class='sender'>{0}</span>" + "<span class='time'>{1}</span>" + "<div class='operationContainer' style='display:none;'>" + "</div>" + "</div>" + "<div class='messageContent'>{2}" + "</div>", System.GetUserInfo().Nickname, DateToString(new Date()), content);
        This.GetDocument().body.appendChild(msgDiv);
        if (m_AutoScroll) msgDiv.scrollIntoView();
        return msgDiv;
    }
    
    var m_AutoScroll = false;
    
    This.SetAutoScroll = function(scroll)
    {
        m_AutoScroll = scroll;
    }
    
    This.GetAutoScroll = function()
    {
        return m_AutoScroll;
    }
    
    This.ShowError = function(msg, temp)
    {
        temp.childNodes[1].innerHTML = String.format("<span style='color:Red; '>{0}</span>", System.ReplaceHtml(msg));
    }
    
    This.AddMessage = function(msg, temp, addope, showtitle)
    {
        if (addope == undefined) addope = true;
        if (showtitle == undefined) showtitle = true;
    
        msg.Content = msg.Content.replace(/\x5BFILE\x3A[^\t\n\r\f\v\x5B\x5D]+\x5D/ig,
        function(val)
        {
            var file = val.substr(6, val.length - 7);
            return String.format(
            m_FileHtmlFormat, System.Path.GetFileName(unescape(file)), file);
        });
    
        msg.Content = msg.Content.replace(/\x5BA\x3A[^\t\n\r\f\v\x5B\x5D]+\x5D/ig,
        function(val)
        {
            var file = val.substr(3, val.length - 4);
            return String.format("<a target='_blank' href='{0}' onclick='return window.OpenLink(this.href);'>", unescape(file));
        });
    
        msg.Content = msg.Content.replace(/\x5B\x2FA\x5D/ig,
        function(val)
        {
            return "</a>";
        });
    
        var msgDiv = This.GetDocument().createElement("DIV");
        msgDiv.className = "message";
        msgDiv.innerHTML = String.format("<div class='messageTitle'>" + "<span class='sender'>{0}</span>" + "<span class='time'>{1}</span>" + "<div class='operationContainer' style='display:none;'>" + "</div>" + "</div>" + "<div class='messageContent'>{2}" + "</div>", msg.SenderNickname, DateToString(msg.CreatedTime), msg.Content);
        msgDiv.onmousemove = function()
        {
            this.firstChild.childNodes[2].style.display = "inline";
        }
        msgDiv.onmouseout = function()
        {
            this.firstChild.childNodes[2].style.display = "none";
        }
    
        if (!showtitle) msgDiv.firstChild.style.display = "none";
    
        if (addope && config.Operation != undefined)
        {
            for (var i in config.Operation)
            {
                try
                {
                    var ope = config.Operation[i];
                    (function(text, cmd, msg, msgHtml, callback)
                    {
                        var opeLink = This.GetDocument().createElement("A");
                        opeLink.className = "operation";
                        opeLink.innerHTML = text;
                        opeLink.onclick = function()
                        {
                            callback(cmd, msg, msgHtml);
                        }
                        var opeContainer = msgDiv.firstChild.childNodes[2];
                        opeContainer.appendChild(opeLink);
                    })(ope.Text, ope.Command, msg, msgDiv.innerHTML,
                    function(cmd, msg, msgHtml)
                    {
                        This.OnCommand.Call(cmd, msg, msgHtml);
                    });
                }
                catch(ex)
                {}
            }
        }
        if (temp == undefined || temp == null)
        {
            This.GetDocument().body.appendChild(msgDiv);
        }
        else
        {
            This.GetDocument().body.replaceChild(msgDiv, temp);
        }
        if (m_AutoScroll) msgDiv.scrollIntoView();
        msgDiv = null;
    }
    
    This.GetWindow().OpenLink = function(href)
    {
        return confirm(String.format("您确定要打开链接 \"{0}\"", href));;
    }
    
    This.GetWindow().SaveFile = function(filePath)
    {
        var folderBrowser = new CommonDialog.FolderBrowserDialog({
            Left: 0,
            Top: 0,
            Width: 400,
            Height: 400,
            Css: 'window',
            BorderWidth: 6,
            Resizable: true,
            HasMinButton: false,
            Title: {
                Height: 18,
                InnerHTML: '另存为'
            },
            Desription: "请选择文件夹:",
            Root: System.GetFullPath("Home")
        });
    
        folderBrowser.OnSelected.Attach(
        function(fb)
        {
            var path = fb.GetSelectedFolderPath();
            if (path != null)
            {
                This.GetTopContainer().Waiting("正在保存文件...");
    
                IO.File.Copy(
                function()
                {
                    This.GetTopContainer().Completed();
                },
                function(error)
                {
                    This.GetTopContainer().Completed();
                    CommonDialog.Alert(error.Message, This.GetTopContainer());
                },
                [filePath], path);
            }
        });
        folderBrowser.ShowDialog(This.GetTopContainer(), 'center', 0, 0);
        folderBrowser.Focus();
    }
    
    This.GetWindow().OpenFile = function(filePath)
    {
        System.Exec(EmptyCallback, alert, "OpenWith.js", filePath, false);
    }
    
    This.ScrollToTop = function()
    {
        This.GetDocument().body.scrollTop = 0;
        //This.GetDocument().body.firstChild.scrollIntoView();
    }
    
    This.ScrollToBottom = function()
    {
        var index = this.GetDocument().body.childNodes.length - 1;
        This.GetDocument().body.childNodes[index].scrollIntoView();
    }

    This.Resize(width,height);
}




function MessageEditor(config)
{
    var This = this;
    var OwnerForm = this;
    
config.StyleSheet = Module.GetResourceUrl("EditorCss.css");
    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    RichEditor.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "MessageEditor"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.CreateFileHtml = function(paths)
    {
        var fs = [];
        for (var i in paths)
        {
            var html = "<div contenteditable='false' class='message_file'>" + String.format("[FILE:{0}]", paths[i]) + "</div>";
            fs.push(html);
        }
        return "<br/>" + fs.join("<br/>") + "<br/>";
    }

    This.Resize(width,height);
}




function ChatPanelContainer(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=615;
    config.Height=540;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "ChatPanelContainer"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_Tab = new Controls.SimpleTabControl({"Left":0,"Top":0,"Width":615,"Height":540,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"simple_tab_gred","Tabs":[{"Text":"起始页","Width":100,"ID":"ID000179","IsSelected":true}],"BorderWidth":1});
    
    
    
    m_Tab.OnSelectedTab.Attach(
        function(index,preIndex)
        {
            
        }
    )

    
    var m_Browser_Config={"Left":8,"Top":8,"Width":596,"Height":498,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":m_Tab.GetPanel(0),"Text":"","Css":"control"};
    
            
    
    var m_Browser = new Browser(m_Browser_Config);
    
    
    

    var _panels = [];
    var _maxTabCount = 4;
    
    This.StartChat = function(theme)
    {
        var index = m_Tab.Find(function(tc)
        {
            return tc.ID == theme.ID
        });
    
        if (index < 0)
        {
            if (_panels.length >= _maxTabCount)
            {
                _panels[0].Stop();
                This.EndChat(_panels[0].GetThemeInfo().ID);
            }
    
            var tabConf = {
                Text: theme.Subject.length <= 10 ? theme.Subject: theme.Subject.substr(0, 10) + "...",
                ID: theme.ID
            };
    
            m_Tab.Insert( - 1, tabConf);
            index = m_Tab.Find(function(tc)
            {
                return tc.ID == theme.ID
            });
    
            if (index < 0) return;
    
            var panel = m_Tab.GetPanel(index);
    
            var chatPanel = new ChatPanel({
                Left: 8,
                Top: 8,
                Width: panel.GetClientWidth() - 16,
                Height: panel.GetClientHeight() - 16,
                Parent: panel,
                AnchorStyle: Controls.AnchorStyle.All,
                Theme: theme,
                Container: This
            });
    
            _panels.push(chatPanel);
        }
        m_Tab.Select(index);
    }
    
    This.EndChat = function(id)
    {
        var i = 0;
        for (; i < _panels.length && _panels[i].GetThemeInfo().ID != id; i++);
        if (i < _panels.length) _panels.splice(i, 1);
    
        var index = m_Tab.Find(function(tc)
        {
            return tc.ID == id;
        });
        if (index == -1) return;
        m_Tab.Remove(index);
    }

    This.Resize(width,height);
}

function NewThemeForm()
{
    var This = this;
    var OwnerForm = this;
    
    var config = {"Left":8,"Top":5,"Width":525,"Height":485,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":false,"HasMaxButton":false,"Resizable":true,"Title":{"InnerHTML":"新建主题"}};
    

    Window.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "NewThemeForm"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var label1 = new Controls.Label({"Left":11,"Top":15,"Width":44,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"类别：","Css":"label"});
    
    

    
    var m_EditCategory = new Controls.TextBox({"Left":61,"Top":11,"Width":443,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"textbox","BorderWidth":1});
    
    m_EditCategory.GetTextBoxDom().readOnly = true;
    if (CurrentCategoryID != 0)
    {
        m_EditCategory.SetText(Categories[CurrentCategoryID].Name);
    }

    
    var label3 = new Controls.Label({"Left":11,"Top":45,"Width":43,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"标题：","Css":"label"});
    
    

    
    var m_EditSubject = new Controls.TextArea({"Left":11,"Top":66,"Width":493,"Height":65,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"textarea","BorderWidth":1});
    
    

    
    var m_BtnOK = new Controls.Button({"Left":438,"Top":423,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"确定","Css":"button_default"});
    
    
    
    m_BtnOK.OnClick.Attach(
        function(btn)
        {
            if (m_EditSubject.GetText().length == 0)
            {
                CommonDialog.Alert("标题不能为空！", This);
                return;
            }
            
            if (m_EditSubject.GetText().length > 128)
            {
                CommonDialog.Alert("标题不能超过128个字符！", This);
                return;
            }
            
            if (m_EditContent.GetValue().length > 102400)
            {
                CommonDialog.Alert("内容不能超过102400个字符！", This);
                return;
            }
            
            function NewTheme_Error(ex)
            {
                This.Completed();
                CommonDialog.Alert(ex, This);
            }
            
            function NewTheme_Complete(ret)
            {
                This.Completed();
                Instance.StartChat(ret.Theme);
                This.Close();
            
            }
            
            This.Waiting();
            NewTheme(NewTheme_Complete, NewTheme_Error, CurrentCategoryID, m_EditSubject.GetText(), TranslateMessage(m_EditContent.GetValue()));
        }
    )

    
    var label6 = new Controls.Label({"Left":12,"Top":142,"Width":60,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"内容：","Css":"label"});
    
    

    
    var m_EditContent_Config={"Left":13,"Top":161,"Width":490,"Height":255,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            m_EditContent_Config.BorderWidth = 1;
    
    var m_EditContent = new MessageEditor(m_EditContent_Config);
    
    
    

    var m_Task = null;
    if(config.HasMinButton)
    {
        m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
        This.OnClosed.Attach(
            function()
            {
                Taskbar.RemoveTask(m_Task);
            }
        );
    }
    var m_MaxTabCount = 4;
    var m_ChatForms = {};
}




function BrowserFrame(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "BBS_Browser";
config.BorderWidth = 1;
    
    var width = config.Width, height = config.Height;
    config.Width=599;
    config.Height=507;

    Controls.Frame.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "BrowserFrame"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    

    This.Resize(width,height);
}




function Browser(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=589;
    config.Height=496;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "Browser"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_EditPath = new Controls.TextBox({"Left":0,"Top":0,"Width":516,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"BBS_BrowserPath","BorderWidth":1});
    
    This.GetDom().onkeydown = function(evt)
    {
        if (evt == undefined) evt = event;
        if (evt.keyCode == 13) This.Navigate(m_EditPath.GetText());
    }

    
    var m_Browser_Config={"Left":0,"Top":27,"Width":589,"Height":469,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            
    
    var m_Browser = new BrowserFrame(m_Browser_Config);
    
    

    
    var label3 = new Controls.Label({"Left":520,"Top":5,"Width":62,"Height":14,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"设为起始页","Css":"BBS_LabelLink"});
    
    label3.GetDom().onclick = function()
    {
        m_ImgWaiting.SetVisible(true);
        label3.SetVisible(false);
        try
        {
            Management.Management.UpdateUserInfo(
            function(newInfo)
            {
                This.Navigate(newInfo.HomePage);
                m_ImgWaiting.SetVisible(false);
                label3.SetVisible(true);
            },
            function(error)
            {
                m_ImgWaiting.SetVisible(false);
                label3.SetVisible(true);
                CommonDialog.Alert(error.Message, This);
            },
            System.GetUser(), {
                HomePage: m_EditPath.GetText()
            });
        }
        catch(ex)
        {
            m_ImgWaiting.SetVisible(false);
            label3.SetVisible(true);
            CommonDialog.Alert(ex.message, This);
        }
    }

    
    var m_ImgWaiting = new Controls.Control({"Left":541,"Top":4,"Width":16,"Height":16,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"http://","Css":"BBS_LabelLink_Setting"});
    
    
    
    m_ImgWaiting.OnResized.Attach(
        function(btn)
        {
            
        }
    )
    

    var _url = System.GetUserInfo().HomePage;
    if (_url == "") _url = window.location.host + System.Path.GetDirectoryName(window.location.pathname) + "/bbs/help.htm";
    if (_url.length < 7 || _url.substr(0, 7).toLowerCase() != "http://") _url = "http://" + _url;
    
    This.Navigate = function(url)
    {
        if (url.length < 7 || url.substr(0, 7).toLowerCase() != "http://")
        {
            m_Browser.Navigate("http://" + url);
        }
        else
        {
            m_Browser.Navigate(url);
        }
    }
    
    m_EditPath.SetText(_url);
    This.Navigate(_url);
    
    m_ImgWaiting.SetVisible(false);

    This.Resize(width,height);
}

